#!/usr/bin/env python
"""
Maildir Web Reader
by Javantea
Sept 27, 2007 - Jan 13, 2013
Major updates continue through Sept 24, 2015
Retired completely Mar 26, 2015

Main page:
	directories on the left
	messages on the top right
	message body in the bottom right

"""
from __future__ import print_function
import os
import mail1
import email.header
import urllib
import tempfile

def sanitize(data):
	if data == None: return '<i>None.</i>'
	if data == '': return '<i>Empty.</i>'
	output = data.replace('&', '&amp;')
	output = output.replace('<', '&lt;')
	output = output.replace('>', '&gt;')
	output = output.replace('"', '&quot;')
	#output = output.replace('<', '&lt;')
	return output
#end def sanitize(data)

def decodeHeaderWeb(h, toSanitize = True, debug=False):
	if h == None:
		if toSanitize: return sanitize(h)
		return ''
	#end if
	if isinstance(h, email.header.Header):
		h = str(h)
	#end if
	if h.strip()[:2] == '=?' or '=?' in h:
		output = decodeQE(h, debug)
	else:
		"""
		junk = False
		for x in h:
			if ord(x) > 127 or ord(x) < 32:
				if x not in '\t\r\n':
					junk = True
				#end if
			#end if
		#next x
		if junk:
			output = 'junk' + repr(h)
		else:
			output = h
		#end if
		"""
		# Seems legit... How could this be a bad choice? Oh yeah.. 
		# TODO: Ensure that we don't get hacked putting a decoder and a sanitizer together. Shit.
		if isinstance(h, bytes):
			output = h.decode('latin-1')
		else:
			output = h
		#end if
	#end if
	if toSanitize:
		return sanitize(output)
	#end if
	return output
#end def decodeHeaderWeb(h)

def drawIndex(path=None):
	"""
	/mail1mod.py
	"""
	if path == None:
		#path = '/home/jvoss/Mail'
		path = os.path.join(os.path.expanduser('~'), 'Mail')
	#end if
	hfile = 'mail1h.txt'
	output  = open(mail1.share_path + hfile).read()
	#output += "<h1>AltSci Maildir Client Demo</h1>"
	# directories on the left
	folders = mail1.readDir(path)
	output += '<div class="dirs" id="dirs">'
	output += '<a style="float:right;" href="/mail1mod.py/logout" onclick="logout(); return false;">Logout</a>'
	output += '<a href="#" onclick="showCompose(); return false;">Compose</a>'
	output += '<table cellpadding="0" cellspacing="2" id="dirtable">'
	dirs = 0
	for cdir in folders['dirs']:
		allInCur = mail1.readDir(os.path.join(path, cdir, 'cur', ''))
		allInCur_len = len(allInCur['files'])
		read_len = 0
		for filename in allInCur['files']:
			if filename[-4:] == ':2,S': read_len += 1
			if filename[-5:] == ':2,RS': read_len += 1
		#next file
		unread_len = allInCur_len - read_len
		output += '<tr><td><a href="/mail1mod.py?dir=' + sanitize(cdir) + '" target="dirf">' + sanitize(cdir) + '</a></td><td>' + str(unread_len) + '</td><td>' + str(allInCur_len) + '</td></tr>\n'
		# subdirs .inbox.directory/
		subdir = '.' + cdir[:-1] + '.directory' + os.sep
		if subdir in folders['hidden dirs']:
			subdirs = mail1.readDir(os.path.join(path, subdir))
			for s in subdirs['dirs']:
				output += '<tr><td><a href="/mail1mod.py?dir=' + sanitize(subdir + s) + '" class="sub" target="dirf">' + sanitize(s) + '</a></td></tr>\n'
				dirs += 1
			#next s
		#end if
		dirs += 1
	#next cdir
	#for a in d['hidden']: output += a
	output += '</table>'
	if dirs < 16:
		output += '<br />' * (16 - dirs)
	#end if
	output += '<br /><form action="/mail1mod.py" method="GET" id="searchForm" target="dirf">\n'
	output += '<input type="hidden" name="dir" value="badkm" />\n'
	output += '<input type="text" name="search" value="Search..." style="width:100%;" onclick="if(this.value==\'Search...\')this.value=\'\';" /></form>\n'
	output += '</div>\n'
	output += '<iframe id="dirf" name="dirf" class="msgsf" src="/mail1mod.py?dir=badkm/"></iframe>\n'
	output += '<iframe id="msgf" name="msgf" class="msgf" src="about:blank"></iframe>\n'
	output += '</body></html>'
	return output
#end def drawIndex(path)

def drawDir(path = None, currentDir = '', page = 0):
	if path == None:
		#path = '/home/jvoss/Mail'
		path = os.path.join(os.path.expanduser('~'), 'Mail')
	#end if
	pagesize = 220

	# TODO: Catch error if we don't have access to it.
	tmpf = os.path.join(tempfile.gettempdir(), 'mail1_junk.txt')
	if os.path.exists(tmpf):
		junkFiles = open(tmpf, 'r').read().split('\n')
	else:
		junkFiles = []
	#end if

	# messages on the top right
	output = '<div class="msgsd pre">\n'
	output += '<table cellpadding="0" cellspacing="0">\n'
	output += '<tr><th>Subject</th><th>From</th><th>Date</th></tr>\n'
	messages = mail1.readDir(os.path.join(path, currentDir, 'cur'))
	show = messages['files']
	show.sort(reverse = True)
	show = show[page*pagesize:(page+1)*pagesize]
	for message in show:
		fname = os.path.join(path, currentDir, 'cur', message)
		f = open(fname, 'r')
		# This takes too long, need to use index or something...
		msg = mail1.message_from_binary_file(f)
		#msg = {'subject':'', 'from':'', 'date':''}
		subject = msg['subject']
		lsubject = decodeHeaderWeb(subject, False, True)
		mfrom = msg['from']
		fenc = decodeHeaderWeb(mfrom, False)
		mfrom = sanitize(fenc)
		mto = msg['to']
		tenc = decodeHeaderWeb(mto, False)
		mto = sanitize(tenc)
		date = sanitize(msg['date'])
		spam_level = len(msg.get('X-Spam-Level', ''))
		if fname in junkFiles: spam_level = 10
		spf = msg.get('Received-SPF', '')
		if spf[:8].lower() == 'softfail':
			spam_level = max(spam_level, 5)
		elif spf[:4].lower() == 'none':
			spam_level = max(spam_level, 4)
		#end if
		spam_icon = '<span style="background-color:' + mail1.spam_color[min(spam_level, len(mail1.spam_color)-1)] + '">&nbsp;</span>'
		smfrom = mfrom
		if len(lsubject) > 50:
			ssubject = sanitize(lsubject[:50]) + '...'
		else:
			ssubject = sanitize(lsubject)
		#end if
		lsubject = sanitize(lsubject)
		if len(fenc) > 20:
			smfrom = sanitize(fenc[:20]) + '...'
		else:
			smfrom = mfrom
		#end if
		if len(tenc) > 20:
			smto = sanitize(tenc[:20]) + '...'
		else:
			smto = mto
		#end if
		if 'sent-mail' in currentDir:
			saddr = smto
			taddr = mto
		else:
			saddr = smfrom
			taddr = mfrom
		#end if
		output += '<tr><td><div class="s"><a href="mail1mod.py?dir=' + currentDir + '&msg=' + message + '" title="' + lsubject + '" target="msgf">' + spam_icon + ssubject + '</a></div></td><td><div class="f" title="' + taddr + '">' + saddr + '</div></td><td><div class="d" title="' + date + '">' + date + '</div></td></tr>\n'
	#next message
	output += '</table>'
	output += '</div>\n'
	output += '<a href="mail1mod.py?dir=' + currentDir + '&page=' + sanitize(str(int(page) + 1)) + '">Next page</a>\n'
	output += '</body>'
	output += '</html>'
	return output
#end def drawDir()


def drawSearch(path = None, search = '', currentDir = '', page = 0):
	if path == None:
		#path = '/home/jvoss/Mail'
		path = os.path.join(os.path.expanduser('~'), 'Mail')
	#end if
	pagesize = 220

	# TODO: Catch error if we don't have access to it.
	tmpf = os.path.join(tempfile.gettempdir(), 'mail1_junk.txt')
	if os.path.exists(tmpf):
		junkFiles = open(tmpf, 'r').read().split('\n')
	else:
		junkFiles = []
	#end if

	# messages on the top right
	output = '<div class="msgsd pre">\n'
	output += '<table cellpadding="0" cellspacing="0">\n'
	output += '<tr><th>Subject</th><th>From</th><th>Date</th></tr>\n'
	messages = mail1.readDir(os.path.join(path, currentDir, 'cur'))
	show = messages['files']
	show.sort(reverse = True)
	i = 0
	for message in show:
		if i < page*pagesize or i >= (page+1)*pagesize:
			break
		#end if
		
		fname = os.path.join(path, currentDir, 'cur', message)
		f = open(fname, 'rb')
		# This takes too long, need to use index or something...
		msg = message_from_binary_file(f, fname)
		#msg = {'subject':'', 'from':'', 'date':''}
		subject = msg['subject']
		lsubject = decodeHeaderWeb(subject, False)
		mfrom = msg['from']
		date = msg['date']
		#try:
		#	fenc = email.header.decode_header(mfrom)[0][0]
		#except:
		#	pass
		#end try
		#mfrom = sanitize(mfrom)
		fenc = decodeHeaderWeb(mfrom, False)
		
		# Not yet full text search.
		found = False
		if lsubject and search in lsubject: found = True
		if not found and subject and search in subject: found = True
		if not found and mfrom   and search in   mfrom: found = True
		if not found and date    and search in    date: found = True
		if not found:
			continue
		#end if
		#fenc = fenc.decode('latin-1')
		#if type(fenc) == str:
		#	fenc = str(fenc.__class__)
		#end if
		mfrom = sanitize(fenc)
		date = sanitize(date)
		spam_level = len(msg.get('X-Spam-Level', ''))
		if fname in junkFiles: spam_level = 10
		spf = msg.get('Received-SPF', '')
		if spf[:8].lower() == 'softfail':
			spam_level = max(spam_level, 5)
		elif spf[:4].lower() == 'none':
			spam_level = max(spam_level, 4)
		#end if
		spam_icon = '<span style="background-color:' + mail1.spam_color[min(spam_level, len(mail1.spam_color)-1)] + '">&nbsp;</span>'
		smfrom = mfrom
		if len(lsubject) > 50:
			ssubject = sanitize(lsubject[:50]) + '...'
		else:
			ssubject = sanitize(lsubject)
		#end if
		lsubject = sanitize(lsubject)
		if len(fenc) > 20:
			smfrom = sanitize(fenc[:20]) + '...'
		else:
			smfrom = mfrom
		#end if
		output += '<tr><td><div class="s"><a href="mail1mod.py?dir=' + currentDir + '&msg=' + message + '" title="' + lsubject + '" target="msgf">' + spam_icon + ssubject + '</a></div></td><td><div class="f" title="' + mfrom + '">' + smfrom + '</div></td><td><div class="d" title="' + date + '">' + date + '</div></td></tr>\n'
		i += 1
	#next message
	output += '</table>'
	output += '</div>\n'
	output += '<a href="mail1mod.py?dir=' + currentDir + '&page=' + sanitize(str(int(page) + 1)) + '">Next page</a>\n'
	output += '</body>'
	output += '</html>'
	return output
#end def drawSearch(path = None, search = '', currentDir = '', page = 0)

def drawMsg(path = None, currentDir = '', currentMessage = '', add = '', use_part = None):
	if path == None:
		#path = '/home/jvoss/Mail'
		path = os.path.join(os.path.expanduser('~'), 'Mail')
	#end if
	html = (add == 'html')
	tryCleanHtml = False
	useTextHtml = False
	if add == 'clean_html':
		html = True
		tryCleanHtml = True
	elif add == 'text_html':
		html = True
		useTextHtml = True
	#end if
	img  = (add == 'img')
	app  = (add == 'app')
	gpg  = (add == 'gpg')

	if (('..' + os.sep) in currentDir) or (('..' + os.sep) in currentMessage):
		return "No directory traversal please..."
	#end if
	
	mime_ext = {'image/jpeg':'jpg', 'image/png':'png', 'image/gif':'gif', 'application/pdf':'pdf',
	'application/zip':'zip', 'text/plain':'txt', 'application/pgp-encrypted':'txt'}
	# TODO: disallow certain extensions
	#disallow_ext = ['exe', 'sh', 'py', 'bin']

	# message body on the bottom right
	output = '<div class="msgd">\n'
	
	fname = os.path.join(path, currentDir, 'cur', currentMessage)
	if not os.path.exists(fname):
		currentMessage2 = currentMessage + ':2,S'
		if not os.path.exists(os.path.join(path, currentDir, 'cur', currentMessage2)):
			return output + 'File not found. Please try again.</div>'
		else:
			currentMessage = currentMessage2
			fname = os.path.join(path, currentDir, 'cur', currentMessage)
		#end if
	#end if
	
	"""
	We don't have write access, so no go. This is easy but silly.
	if currentMessage[-4:] != ':2,S' and currentMessage[-5:] != ':2,RS':
		currentMessage2 = currentMessage + ':2,S'
		# It's been seen, so rename it (this is way weird)
		os.rename(path + '/' + currentDir + '/cur/' + currentMessage, path + '/' + currentDir + '/cur/' + currentMessage2)
		currentMessage = currentMessage2
	#end if
	"""
	
	f = open(fname, 'rb')
	#f.seek(0, 2)
	#file_size = f.tell()
	#f.seek(0, 0)
	msg = message_from_binary_file(f, fname)
	subject = msg['subject']
	ssubject = decodeHeaderWeb(subject)
	mfrom = decodeHeaderWeb(msg['from'])
	date = decodeHeaderWeb(msg['date'])
	mto = decodeHeaderWeb(msg['to'])
	message_id = None
	if 'Message-ID' in msg:
		message_id = decodeHeaderWeb(msg['Message-ID'])
	#end if
	mcc = None
	if 'cc' in msg:
		mcc = decodeHeaderWeb(msg['cc'])
	#end if
	from_filter = genFilter(msg['from'], 'From')
	to_filter = genFilter(msg['to'], 'To')

	output += '<a href="#" onclick="if (window.parent && window.parent.reply) window.parent.reply(document); else alert(\'fixme \' + window.parent.toString());">Reply</a>\n'
	output += '<a href="mail1mod.py?' + urllib.urlencode({'dir':currentDir, 'msg': currentMessage, 'junk':'1'}) + '">Spam</a>\n'
	output += '<div class="msgh pre" onclick="if(headers.style.display==\'block\') headers.style.display=\'none\'; else headers.style.display=\'block\';">\n<div><b>Subject:</b> <span id="s">' + ssubject + '</span></div>\n'
	output += '<div><b>From:</b> <span id="f">' + mfrom + '</span> <span class="filter" onclick="this.innerHTML=unescape(\'' + from_filter + '\'); return false;">Filter</span></div>\n'
	output += '<div><b>To:</b> <span id="t">' + mto + '</span> <span class="filter" onclick="this.innerHTML=unescape(\'' + to_filter + '\'); return false;">Filter</span></div>\n'
	if mcc:
		output += '<div id="cc"><b>Cc:</b> <span id="c">' + mcc + '</span></div>\n'
	#end if
	output += '<div><b>Date:</b> <span id="d">' + date + '</span></div>\n'
	if message_id:
		output += '<div><b>Message-ID:</b> <span id="messageid">' + message_id + '</span></div>\n'
	#end if
	output += '</div>\n'
	#return output
	output += '<div class="pre">\n'
	i = 0
	output += '<div id="headers" class="msgh2" style="display:none;">\n'
	for (k, v) in msg.items():
		output += sanitize(k + ':' + v.decode('latin-1')) + '<br />\n'
	#next k,v
	output += '</div>\n'
	output += '</div>\n'
	output += '<div class="pre" id="b">\n'
	signed = False
	encrypted = False
	for part in msg.walk():
		#if part.get_size() > 10000000:
		#	output = ''
		#end if
		ctype = part.get_content_type()
		f_raw = part.get_filename()
		if f_raw == None: f_raw = ''
		if f_raw != '':
			f_decoded = email.header.decode_header(f_raw)
			if f_decoded[0][1] != None:
				f_uni = f_decoded[0][0].decode(f_decoded[0][1]).encode('utf-8')
				filename = sanitize(f_uni)
			else:
				filename = sanitize(f_decoded[0][0])
			#end if
		else:
			filename = None
		#end if
		#ssize = str(part.get_size())
		ssize = ''
		output += '<!--' + sanitize(ctype) + '-->\n' #'<br />\n'
		#fname = part.get_param('name')
		#output += 'dir:' + sanitize(repr(dir(part))) + '<br />\n'
		#output += 'name:' + sanitize(repr(part.get_param('name'))) + '<br />\n'
		#continue
		if ctype == 'multipart/related' or ctype == 'multipart/mixed' or ctype == 'multipart/alternative' or ctype == 'multipart/signed' or ctype == 'multipart/encrypted':
			if ctype == 'multipart/signed':
				signed = True
			#end if
			if ctype == 'multipart/encrypted':
				encrypted = True
			#end if
			pass
			#output += '<div style="border:1px solid black;">\n'
			#for partb in part.walk():
			#	output += sanitize(str(part))
			#	output += '<hr />\n'
			#next partb
			#output += '</div>\n'
		elif ctype[:6] == 'image/':
			ext = 'bin'
			if ctype in mime_ext: ext = mime_ext[ctype]
			if filename == None: filename = ('image%03i' % i) + '.' + ext
			if img and (str(i) == use_part):
				return (ctype, part.get_payload(decode=True), None)
			else:
				output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&img=' + str(i) + '"><img src="/images/image2.png" border="0" /> ' + str(img) + ' ' + str(use_part) + ' ' + str(i) + ' ' + filename + '</a><br />\n'
			#end if
		elif ctype[:12] == 'application/' or filename != None:
			ext = 'bin'
			if encrypted:
				ext = 'gpg'
			#end if
			if ctype in mime_ext: ext = mime_ext[ctype]
			if filename == None: filename = ('file%03i' % i) + '.' + ext
			if app and (str(i) == use_part):
				return (ctype, part.get_payload(decode=True), filename)
			else:
				try:
					filenameu = unicode(filename)
				except UnicodeDecodeError:
					filenameu = repr(filename)
				#end try 
				output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&app=' + str(i) + '"><img src="/images/binary.png" border="0" /> ' + filenameu + '</a> ' + ssize + '<br />\n'
			#end if
		elif ctype[:6] == 'audio/':
			ext = 'bin'
			if ctype in mime_ext: ext = mime_ext[ctype]
			if filename == None: filename = ('audio%03i' % i) + '.' + ext
			if app and (str(i) == use_part):
				return (ctype, part.get_payload(decode=True), filename)
			else:
				output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&app=' + str(i) + '"><img src="/images/binary.png" border="0" /> ' + filename + '</a><br />\n'
			#end if
		elif ctype == 'text/html':
			if html == True:
				# Be very careful to ensure this does not get execed.
				pdata = part.get_payload(decode=True)
				ct = part.get('Content-Type')
				charset = 'ascii'
				if ct and 'charset=' in ct:
					charset = ct[ct.index('charset=')+8:]
					try:
						pdata = pdata.decode(charset)
					except LookupError as le:
						# TODO: warning.
						pass
					except UnicodeDecodeError as ude:
						# TODO: warning.
						pass
					#end try
				#end if
				try:
					uni_input = part.get_payload(decode=True).decode(charset)
				except LookupError:
					# FIXME: This could be better handled.
					uni_input = part.get_payload(decode=True).decode('ascii')
				except UnicodeDecodeError:
					# FIXME: This could be better handled.
					uni_input = part.get_payload(decode=True).decode('latin')
				#end try
				if tryCleanHtml:
					output += clean_html(uni_input)
				elif useTextHtml:
					output += sanitize(text_html(uni_input)).replace('\n', '<br />\n')
				else:
					output += sanitize(uni_input).replace('\n', '<br />\n')
				#end if
				output += '<hr />\n'
			else:
				if filename == None: filename = ''
				output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&html=1">html ' + filename + '</a><br />\n'
				output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&text_html=1">html text ' + filename + '</a><br />\n'
				output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&clean_html=1">html clean ' + filename + '</a><br />\n'
			#end if
		elif ctype == 'text/plain':
			if (app or gpg) and (str(i) == use_part):
				ext = 'txt'
				if filename == None: filename = ('file%03i' % i) + '.' + ext
				if gpg:
					# RFC3156 wants unencoded with headers.
					output = '\n'.join([': '.join(a) for a in part.items()]) + '\n\n'
					output += part.get_payload(decode=False)
				else:
					output = part.get_payload(decode=True)
				#end if
				return (ctype, output, filename)
			if html == False:
				#.decode()
				pdata = part.get_payload(decode=True)
				ct = part.get('Content-Type')
				charset = 'utf-8'
				if ct and ('charset=' in ct or 'charset =' in ct):
					if 'charset=' in ct:
						charset = ct[ct.index('charset=')+8:].strip()
					elif 'charset =' in ct:
						charset = ct[ct.index('charset =')+9:].strip()
					#end if
					if ';' in charset: charset = charset[:charset.index(';')]
					if '"' in charset: charset = charset.replace('"','')
				#end if
				try:
					pdata = pdata.decode(charset)
				except LookupError as le:
					# TODO: warning.
					warning = 'Lookup Warning: ' + str(le) + '\n'
					pdata = warning + pdata.decode('utf-8', errors='ignore')
				except UnicodeDecodeError as ude:
					# TODO: warning.
					warning = 'Unicode Warning: ' + str(ude) + '\n'
					pdata = warning + pdata.decode(charset, errors='ignore')
				#end try
				# Also be very careful to ensure this does not get execed.
				#if msg['Content-Transfer-Encoding'] == 'base64':
				#	pdata2 = pdata.split('\n\n', 1)
				#	pdata = pdata2[0] + base64.b64decode(pdata2[1])
				#end if
				#output += charset + sanitize(repr(pdata))
				# TODO: make sure this try is actually necessary.
				try:
					output += sanitize(pdata).replace('\n', '<br />\n')
				except UnicodeDecodeError as eex:
					output += 'Error with data: ' + str(eex) + ' <br />\n' + repr(part.get_params()) + ' <br />\n' + sanitize(repr(pdata))
				# + "(" + str(charset) + " " + repr(part.get_all('Content-Type')) + ", " + str(type(pdata)) + ")").replace('\n', '<br />\n')
				output += '<hr />\n'
				if signed:
					output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&gpg=' + str(i) + '">gpg</a><br />\n'
				#end if
				output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&app=' + str(i) + '">plaintext</a><br />\n'
			else:
				output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '">text</a><br />\n'
			#end if
		else:
			try:
				output += sanitize(str(part)).replace('\n', '<br />\n')
			except UnicodeDecodeError:
				output += sanitize(str(part).decode('utf-8')).replace('\n', '<br />\n')
			#end try
			output += '<hr />\n'
		#end if
		if len(output) > 1000000:
			output += "<div>Debug: It's a big one</div>\n"
		#end if
		i += 1
	#next part
	output += '</div>\n' # pre
	#end if
	output += '</div>\n'
	
	output += '</body>'
	output += '</html>'
	return output
#end def drawMsg()

def main(path = None, currentDir = '', currentMessage = '', add='', use_part=None, page = 0, filter_value=None, search=None):
	# inbox 1190764104.21519.bCHn8:2,S
	#path = '/home/jvoss/Mail'
	#currentDir = 'inbox'
	#currentMessage = '1190764104.21519.bCHn8:2,S'
	if path == None:
		#path = '/home/jvoss/Mail'
		path = os.path.join(os.path.expanduser('~'), 'Mail')
	#end if
	if search:
		hfile = 'mail2h.txt'
		output  = open(share_path + hfile).read()
		# TODO: Directory.
		if not currentDir:
			currentDir = 'badkm'
		#end if
		return output + drawSearch(path, search, currentDir, page)
	#end if
	
	# Session vars
	if currentDir == '':
		return drawIndex(path)
	#end if

	if filter_value != None:
		return mail1.filterDir('list_filter1.txt', path, currentDir)
	#end if

	hfile = 'mail2h.txt'
	# TODO: We have to be certain that mail2h.txt is always utf-8!
	output  = open(share_path + hfile, 'r').read()
	
	if currentDir != '' and currentMessage == '':
		return output + drawDir(path, currentDir, page)
	#end if

	msgHtml = drawMsg(path, currentDir, currentMessage, add, use_part)
	# They want to control the whole output.
	if isinstance(msgHtml, tuple):
		#
		return msgHtml
	#end if
	return output + msgHtml
#end def main(path = '/home/$USER/Mail', currentDir = '', currentMessage = '', add='', use_part=None, page = 0, filter_value=None, search=None)

def logout():
	"""
	This does JavaScript. I think it'll work.
	TODO: Invalidate CSRF token as well as the login token which we have yet to 
	write.
	os.getcwd() == '/' in case you're curious...
	"""
	return open(share_path + 'logout','r').read()
#end def logout()

def login(username=None, password=None):
	"""
	Check credentials, give a login cookie (id).
	"""
	if username and password:
		# FIXME: The password check needs to be constant time and stuff.
		return (True, '')
	#end if
	return (False, open(share_path + 'login','r').read())
#end def login(username=None, password=None)

def genFilter(addr, label):
	"""
	A filter really badly designed.
	"""
	if addr == None: addr = ''
	if '<' in addr:
		addr = addr[addr.index('<')+1:]
	#end if
	if '>' in addr:
		addr = addr[:addr.index('>')]
	#end if
	name = addr
	if '@' in name:
		name = name[name.index('@')+1:]
	#end if
	if '.' in name:
		name = name[:name.index('.')]
	#end if
	# Simple encode
	#return '%' + '%'.join([a.encode('hex') for a in ("'" + sanitize(name) + "':{'" + label + "':'" + sanitize(addr) + "', 'dir':'altsci/x'},")])
	# FIXME: utf-8 is actually not acceptable for this.
	return ''.join(['%' + binascii.hexlify(a.encode('utf-8')).decode('ascii') for a in ("" + sanitize(name) + "," + label + "," + sanitize(addr) + ","+ os.path.join("altsci", "x"))])
#end def genFilter(addr, label)

def main(path = None, currentDir = '', currentMessage = '', add='', use_part=None, page = 0, filter_value=None, search=None):
	# inbox 1190764104.21519.bCHn8:2,S
	#path = '/home/jvoss/Mail'
	#currentDir = 'inbox'
	#currentMessage = '1190764104.21519.bCHn8:2,S'
	if path == None:
		#path = '/home/jvoss/Mail'
		path = os.path.join(os.path.expanduser('~'), 'Mail')
	#end if
	if search:
		hfile = 'mail2h.txt'
		output  = open(share_path + hfile).read()
		# TODO: Directory.
		if not currentDir:
			currentDir = 'badkm'
		#end if
		return output + drawSearch(path, search, currentDir, page)
	#end if
	
	# Session vars
	if currentDir == '':
		return drawIndex(path)
	#end if

	if filter_value != None:
		return mail1.filterDir('list_filter1.txt', path, currentDir)
	#end if

	hfile = 'mail2h.txt'
	output  = open(share_path + hfile).read()
	
	if currentDir != '' and currentMessage == '':
		return output + drawDir(path, currentDir, page)
	#end if

	msgHtml = drawMsg(path, currentDir, currentMessage, add, use_part)
	# They want to control the whole output.
	if isinstance(msgHtml, tuple):
		#
		return msgHtml
	#end if
	return output + msgHtml
#end def main(path = '/home/$USER/Mail', currentDir = '', currentMessage = '', add='', use_part=None, page = 0, filter_value=None, search=None)

if __name__ == '__main__':
	Maildir = os.path.join(os.path.expanduser('~'), 'Mail')
	
	print(main(Maildir, 'badkm', filter=True))
#end if

