#!/usr/bin/env python
"""
AltSci Maildir Gtk+3 Demo
by Javantea
Sept 19, 2013 Original 20-30 lines of code
Nov 28, 2014 Version 0.1

mail1.py is going to have a gtk frontend. Why? GnuPG.

TODO:
Send
Persist data in closed composer
Save attachments
GnuPG reading, signing, and encrypting
Better formatting of messages.
More pages
Remote access to files over SSH or HTTPS without SSHFS.
Indexing headers
"""
from __future__ import print_function

import pygtk
pygtk.require('2.0')
import gtk
import mail1
import email
import os
import urllib
import pango
import stat
import zlib
import reverserepr
import time

def sanitizeGtk(s):
	"""
	We don't need to sanitize in Gtk. Only in HTML.
	"""
	return s
#end def sanitizeGtk(s)

def decodeHeaderGtk(s, toSanitize=True):
	"""
	We don't need to sanitize in Gtk. Only in HTML.
	"""
	return mail1.decodeHeader(s, False)
#end def decodeHeaderGtk(s, toSanitize=False)

def sanitizeGtkButton(s):
	"""
	Buttons use _ as an accelerator marker. We don't use that.
	"""
	return s.replace('_','__')
#end def sanitizeGtkButton(s)

def getIndex(currentDir, page):
	"""
	Setup the index in the hidden dir.
	"""
	home = os.environ['HOME']
	hidden_dir = home + os.sep +'.mail' + os.sep
	index = {}
	if not os.path.exists(hidden_dir):
		return {}
	#end if
	# The odds of a crc32 colliding unintentionally is approximately the 
	# same as the odds of the first 32-bits of sha1 colliding. Collision of 
	# crc32 is just much easier.
	index_name = currentDir.rstrip('/').replace('/', '_') + '.' + str(page) + '.' + ('%08x' % zlib.crc32(currentDir.encode('utf-8')))
	index_path = hidden_dir + index_name + '.idx'
	if os.path.exists(index_path):
		index_lines = open(index_path, 'r').read().split('\n')
		for line in index_lines:
			x = reverserepr.reverseRepr(line)
			# Invalid lines
			if x == None or len(x) != 2: continue
			index[x[0]] = x[1]
		#next line
	#end if
	return index
#end def getIndex(currentDir, page)

def saveIndex(currentDir, page, index):
	# Setup the index in the hidden dir.
	home = os.environ['HOME']
	hidden_dir = home + os.sep +'.mail' + os.sep
	if not os.path.exists(hidden_dir):
		os.mkdir(hidden_dir)
		if not os.path.exists(hidden_dir):
			# Survive inability to create ~/.mail/.
			return False
		#end if
	#end if
	# The odds of a crc32 colliding unintentionally is approximately the 
	# same as the odds of the first 32-bits of sha1 colliding. Collision of 
	# crc32 is just much easier.
	index_name = currentDir.rstrip('/').replace('/', '_') + '.' + str(page) + '.' + ('%08x' % zlib.crc32(currentDir.encode('utf-8')))
	index_path = hidden_dir + index_name + '.idx'
	index_file = open(index_path, 'w')
	for x in index:
		line = repr((x, index[x]))
		index_file.write(line + '\n')
	#next line
	index_file.close()
	#end if
	return True
#end def saveIndex(currentDir, page, index)

def toThreads(msgs_rows, show):
	"""
	msgs_rows is an array of [msg_icon,ssubject,smfrom,date]
	show is a an array of filenames.
	We sort msgs_rows and show and then spit it back out.
	In general the ordering must be in the same chrono order except that 
	older things go under newer things they relate to.
	"""
	skips = []
	output = []
	show_out = []
	for i, msg in enumerate(msgs_rows):
		if i in skips: continue
		output.append(msg)
		show_out.append(show[i])
		(msg_icon,ssubject,smfrom,date) = msg
		for j, msg2 in enumerate(msgs_rows[i+1:]):
			if (i+j+1) in skips: continue
			(msg_icon2,ssubject2,smfrom2,date2) = msg2
			if ssubject == ssubject2 or ssubject == ('Re: ' + ssubject2):
				output.append(msg2)
				show_out.append(show[i+j+1])
				skips.append(i+j+1)
			#end if
		#next j, msg2
	#next i, msg
	return output, show_out
#end def toThreads(msgs_rows, show)

class ComposeWindow:
	def __init__(self, parent, mfrom='', to='', subject='', body='', cc='', bcc='', headers=''):
		"""
		If you give no parent, showAll will call gtk.main(), so don't screw this up.
		"""
		self.parent = parent
		self.mfrom = mfrom
		self.to = to
		self.subject = subject
		self.body = body
		self.cc = cc
		self.bcc = bcc
		self.headers = headers
		
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("destroy", self.notReady)
		self.window.set_title("AltSci Composer")
		self.window.set_size_request(800, 600)
		
		"""
		rowFrom = gtk.HBox(False, 5)
		rowFrom.pack_start(gtk.Label("From:"), expand=False, fill=True)
		self.inputFrom = gtk.Entry()
		rowFrom.pack_start(self.inputFrom, expand=True, fill=True)
		rowTo = gtk.HBox(False, 5)
		rowTo.pack_start(gtk.Label("To:"), expand=False, fill=True)
		self.inputTo = gtk.Entry()
		rowTo.pack_start(self.inputTo, expand=True, fill=True)
		rowCc = gtk.HBox(False, 5)
		rowCc.pack_start(gtk.Label("Cc:"), expand=False, fill=True)
		self.inputCc = gtk.Entry()
		rowCc.pack_start(self.inputCc, expand=True, fill=True)
		rowBcc = gtk.HBox(False, 5)
		rowBcc.pack_start(gtk.Label("Bcc:"), expand=False, fill=True)
		self.inputBcc = gtk.Entry()
		rowBcc.pack_start(self.inputBcc, expand=True, fill=True)
		rowSubject = gtk.HBox(False, 5)
		rowSubject.pack_start(gtk.Label("Subject:"), expand=False, fill=True)
		self.inputSubject = gtk.Entry()
		rowSubject.pack_start(self.inputSubject, expand=True, fill=True)
		"""
		self.bodyView = gtk.TextView()
		self.bodyView.set_wrap_mode(gtk.WRAP_WORD)
		rowSend = gtk.HBox(False, 5)
		sendButton = gtk.Button("Send")
		sendButton.connect("clicked", self.send)
		closeButton = gtk.Button("Close")
		closeButton.connect("clicked", self.close)
		rowSend.pack_start(sendButton, False, False)
		rowSend.pack_end(closeButton, False, False)
		
		self.inputFrom = gtk.Entry()
		self.inputTo = gtk.Entry()
		self.inputCc = gtk.Entry()
		self.inputBcc = gtk.Entry()
		self.inputSubject = gtk.Entry()
		vtable = gtk.Table(6, 2)
		vtable.attach(gtk.Label("From:"), 0, 1, 0, 1, xoptions=gtk.FILL, yoptions=gtk.FILL)
		vtable.attach(self.inputFrom, 1, 2, 0, 1)
		vtable.attach(gtk.Label("To:"), 0, 1, 1, 2, xoptions=gtk.FILL, yoptions=gtk.FILL)
		vtable.attach(self.inputTo, 1, 2, 1, 2)
		vtable.attach(gtk.Label("Cc:"), 0, 1, 2, 3, xoptions=gtk.FILL, yoptions=gtk.FILL)
		vtable.attach(self.inputCc, 1, 2, 2, 3)
		vtable.attach(gtk.Label("Bcc:"), 0, 1, 3, 4, xoptions=gtk.FILL, yoptions=gtk.FILL)
		vtable.attach(self.inputBcc, 1, 2, 3, 4)
		vtable.attach(gtk.Label("Subject:"), 0, 1, 4, 5, xoptions=gtk.FILL, yoptions=gtk.FILL)
		vtable.attach(self.inputSubject, 1, 2, 4, 5, xoptions=gtk.FILL, yoptions=gtk.FILL)
		buttonEditHeaders = gtk.Button("Edit Headers")
		buttonEditHeaders.connect("clicked", self.editHeaders)
		# Left align the button.
		alignEditHeaders = gtk.Alignment(0.0, 0.0)
		alignEditHeaders.add(buttonEditHeaders)
		vtable.attach(alignEditHeaders, 0, 2, 5, 6, xoptions=0, yoptions=gtk.FILL)
		
		body_win = gtk.ScrolledWindow()
		body_win.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		body_win.add_with_viewport(self.bodyView)

		vb = gtk.VBox(False, 5)
		#vb.pack_start(rowFrom, False, False)
		#vb.pack_start(rowTo, False, False)
		#vb.pack_start(rowCc, False, False)
		#vb.pack_start(rowBcc, False, False)
		#vb.pack_start(rowSubject, False, False)
		vb.pack_start(vtable, False, False)
		vb.pack_start(body_win, expand=True, fill=True)
		vb.pack_start(rowSend, False, False)
		self.window.add(vb)
		self.ready = True

	#end def ComposeWindow(mfrom='', to='', subject='', body='', cc='', bcc='', headers='')
	
	def editHeaders(self, x=None):
		print(x)
	#end def editHeaders(self, x)
	
	def send(self, x=None):
		#print(x)
		sfrom = self.inputFrom.get_text()
		to = self.inputTo.get_text()
		cc = self.inputCc.get_text()
		bcc = self.inputBcc.get_text()
		subject = self.inputSubject.get_text()
		mb = self.bodyView.get_buffer()
		start = mb.get_start_iter()
		end = mb.get_end_iter()
		body = mb.get_text(start, end, True)
		#print(sfrom, to, cc, bcc, subject, body)
		mail1.send(sfrom, to, cc, bcc, subject, body, attach=None, headers=None)
	#end def send(self, x)
	
	def close(self, x=None):
		self.window.hide()
	#end def close(self, x)
	
	def showAll(self):
		"""
		Calls self.window.show_all()
		"""
		self.window.show_all()
		if self.parent == None:
			# This might screw stuff up...
			gtk.main()
		#end if
	#end def showAll()
	
	def present(self):
		self.window.present()
	#end def present()
	
	def notReady(self, x=None):
		# TODO: Save the data so we can move on.
		self.ready = False
	#end def notReady(x=None)
	
#end class ComposeWindow

class MaildirWindow:
	"""
	On the left a list of directories.
	Top right, A list of messages in the current directory.
	Bottom right, The current message. Simple, no?
	"""
	def __init__(self, maildir):
		self.messages = []
		self.maildir = maildir
		self.currentDir = 'badkm'
		self.message = None
		self.composer = None
		self.showHeader = False
		self.extra_headers = []
		
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("destroy", lambda w: gtk.main_quit())
		self.window.connect("delete_event", lambda a1,a2:gtk.main_quit())

		self.window.set_title("AltSci Maildir Gtk+2 Demo")
		self.window.set_size_request(800, 600)
		# homogenous, spacing
		hb = gtk.HBox(False, 5)
		self.window.add(hb)
		compose_box = gtk.VBox(False, 5)
		
		compose_button = gtk.Button("Compose")
		compose_button.connect('clicked', self.clickCompose)
		compose_box.pack_start(compose_button, expand=False, fill=True)
		dirs_box = gtk.VBox(False, 5)
		dirs_win = gtk.ScrolledWindow()
		dirs_win.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		dirs_win.add_with_viewport(dirs_box)
		compose_box.pack_start(dirs_win)
		#hb.add(dirs_win)
		
		self.drawIndex(maildir, dirs_box = dirs_box)
		hb.pack_start(compose_box, expand=False, fill=True)
		
		vb = gtk.VBox(False, 5)
		#hb.add(vb)
		hb.pack_start(vb, expand=True, fill=True)
		
		self.unreadPixBuf = gtk.gdk.pixbuf_new_from_file('envelope-closed.png')
		self.readPixBuf   = gtk.gdk.pixbuf_new_from_file('envelope-open.png')
		self.spamPixBuf   = gtk.gdk.pixbuf_new_from_file('x.png')
		self.twoPixBuf    = gtk.gdk.pixbuf_new_from_file('question-mark2.png')
		self.threePixBuf  = gtk.gdk.pixbuf_new_from_file('question-mark3.png')
		self.fourPixBuf   = gtk.gdk.pixbuf_new_from_file('question-mark4.png')
		self.fivePixBuf   = gtk.gdk.pixbuf_new_from_file('question-mark5.png')
		
		self.msgs_win = gtk.ScrolledWindow()
		vb.pack_start(self.msgs_win, expand=True, fill=True)
		self.msg_store = gtk.ListStore(gtk.gdk.Pixbuf, str, str, str)
		self.drawDir(maildir, currentDir = self.currentDir, msg_store = self.msg_store)
		# 101 rows, 3 cols
		self.msg_table = gtk.TreeView(self.msg_store)
		
		rendererText = gtk.CellRendererText()
		rendererImage = gtk.CellRendererPixbuf()
		spam_column = gtk.TreeViewColumn("", rendererImage, pixbuf=0)
		subject_column = gtk.TreeViewColumn("Subject", rendererText, text=1)
		from_column    = gtk.TreeViewColumn("From", rendererText, text=2)
		date_column    = gtk.TreeViewColumn("Date", rendererText, text=3)

		self.msg_table.append_column(spam_column)
		self.msg_table.append_column(subject_column)
		self.msg_table.append_column(from_column)
		self.msg_table.append_column(date_column)
		self.msgs_win.add(self.msg_table)
		self.msg_table.connect('cursor-changed', self.clickMsg)

		message_win = gtk.ScrolledWindow()
		self.message_view = gtk.TextView()
		self.message_view.set_wrap_mode(gtk.WRAP_WORD)
		message_win.add(self.message_view)
		vb.pack_start(message_win, expand=True, fill=True)
		#vb.pack_start(self.message_view, expand=True, fill=True)
	#end def MaildirWindow(maildir)
	
	def clickCompose(self, x):
		if self.composer != None and self.composer.ready:
			#self.composer.showAll()
			self.composer.present()
			return
		#end if
		self.composer = ComposeWindow(self.window)
		self.composer.showAll()
	#end def clickCompose(x)
	
	def drawIndex(self, path, dirs_box):
		"""
		/mail1mod.py
		"""
		als = []
		# directories on the left
		folders = mail1.readDir(path)
		dirs = 0
		for cdir in folders['dirs']:
			alld = mail1.readDir(path + os.sep + cdir + 'cur' + os.sep)
			all_len = len(alld['files'])
			read_len = 0
			for filename in alld['files']:
				if filename[-4:] == ':2,S': read_len += 1
				if filename[-5:] == ':2,RS': read_len += 1
			#next file
			unread_len = all_len - read_len
			als.append(gtk.Button(sanitizeGtkButton(cdir)))
			als[-1].set_relief(gtk.RELIEF_NONE)
			#print(als[-1].get_label())
			# Left aligned
			als[-1].xalign = 0.0
			als[-1].connect('clicked', self.clickDir, cdir)
			dirs_box.pack_start(als[-1], expand=False, fill=True)
			# subdirs .inbox.directory/
			subdir = '.' + cdir[:-1] + '.directory/'
			if subdir in folders['hidden dirs']:
				subdirs = mail1.readDir(path + os.sep + subdir)
				for s in subdirs['dirs']:
					als.append(gtk.Button(sanitizeGtkButton(s)))
					als[-1].set_relief(gtk.RELIEF_NONE)
					als[-1].connect('clicked', self.clickDir, subdir+s)
					# Left aligned
					als[-1].xalign = 0.0
					dirs_box.pack_start(als[-1], expand=False, fill=True)
					dirs += 1
				#next s
			#end if
			dirs += 1
		#next cdir
		return als
	#end def drawIndex(path, dirs_box = None)

	def clickDir(self, widget, path):
		#print("did you click on:", path)
		self.msg_store.clear()
		self.drawDir(self.maildir, path, msg_store = self.msg_store)
		adj = self.msgs_win.get_vadjustment()
		adj.set_value(0)
		adj = self.msgs_win.get_hadjustment()
		adj.set_value(0)
		#return False
	#end def clickDir(widget, path)

	def clickMsg(self, x, add='', use_part=None):
		self.message = None
		cursor = x.get_cursor()
		pos = cursor[0][0]
		#print(x)
		message = self.messages[pos]
		#fname = self.maildir + '/' + self.currentDir + '/cur/' + message
		message_buffer = gtk.TextBuffer()
		self.drawMsg(message_buffer, self.maildir, self.currentDir, message, add, use_part)
		#message_buffer.set_text(message_text)
		self.message_view.set_buffer(message_buffer)
		
		self.message = message
		
		#return False
	#end def clickMsg(x, add='', use_part=None)
	
	def clickHtml(self, tag, widget, event, iterc):
		if event.type != gtk.gdk.BUTTON_PRESS: return
		self.clickMsg(self.msg_table, 'html')
	#end def clickHtml(tag, widget, event, iterc)
	
	def clickHtmlText(self, tag, widget, event, iterc):
		if event.type != gtk.gdk.BUTTON_PRESS: return
		self.clickMsg(self.msg_table, 'text_html')
	#end def clickHtml(tag, widget, event, iterc)
	
	def clickHtmlClean(self, tag, widget, event, iterc):
		if event.type != gtk.gdk.BUTTON_PRESS: return
		self.clickMsg(self.msg_table, 'clean_html')
	#end def clickHtml(tag, widget, event, iterc)

	def clickPlain(self, tag, widget, event, iterc):
		if event.type != gtk.gdk.BUTTON_PRESS: return
		self.clickMsg(self.msg_table)
	#end def clickPlain(tag, widget, event, iterc)

	def clickReply(self, tag, widget, event, iterc):
		if event.type != gtk.gdk.BUTTON_PRESS: return
		#print(x)
		print("Reply")
	#end def clickReply(tag, widget, event, iterc)

	def clickSpam(self, tag, widget, event, iterc):
		if event.type != gtk.gdk.BUTTON_PRESS: return
		#print(x)
		print("Spam")
		message_buffer = gtk.TextBuffer()
		start_iter = message_buffer.get_start_iter()
		curr_iter = start_iter
		
		destDir = self.maildir + os.sep + '.inbox.directory' + os.sep + 'spam1' + os.sep + 'cur'
		
		messagePath = self.maildir + os.sep + self.currentDir + os.sep + 'cur' + os.sep + self.message
		if not messagePath.startswith(self.maildir):
			print('Nothing outside Mail please: ' + messagePath)
			message_buffer.insert(curr_iter, 'Nothing outside Mail please: ' + messagePath)
			self.message_view.set_buffer(message_buffer)
			return False
		#end if
		if ('..' + os.sep) in messagePath:
			print('No directory traversal please: ' + messagePath)
			message_buffer.insert(curr_iter, 'No directory traversal please: ' + messagePath)
			self.message_view.set_buffer(message_buffer)
			return False
		#end if
		message = messagePath.split(os.sep)[-1]
		if not os.path.exists(messagePath):
			print('Error: src does not exist:', messagePath)
			message_buffer.insert(curr_iter, 'Error: src does not exist: ' + messagePath)
			self.message_view.set_buffer(message_buffer)
			return False
		#end if
		if os.path.exists(destDir + os.sep + message):
			print('Error: dest exists:', destDir + os.sep + message)
			message_buffer.insert(curr_iter, 'Error: dest exists: ' + destDir + os.sep + message)
			self.message_view.set_buffer(message_buffer)
			return False
		#end if
		print('renamed', messagePath, destDir + os.sep + message)
		message_buffer.insert(curr_iter, 'renamed ' + messagePath + ' ' + destDir + os.sep + message)
		os.rename(messagePath, destDir + os.sep + message)
		# 644
		try:
			os.chmod(destDir + os.sep + message, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
		except OSError as e:
			print("Warning: chmod failed:", e)
			message_buffer.insert(curr_iter, "Warning: chmod failed: " + str(e))
		#end try
		#moved.append(message)
		
		self.message_view.set_buffer(message_buffer)
		return True
	#end def clickSpam(tag, widget, event, iterc)

	def clickHeader(self, tag, widget, event, iterc):
		# FIXME: These are ridiculous.
		if event.type != gtk.gdk.BUTTON_PRESS: return
		#print(event)
		# Draw the extra headers.
		mb = self.message_view.get_buffer()
		if not mb: return
		if self.showHeader:
			curr_iter = mb.get_iter_at_mark(mb.get_mark("extra_headers"))
			tag_table = mb.get_tag_table()
			bold_tag = tag_table.lookup("bold")
			normal_tag = tag_table.lookup("normal")
			i = 0
			#output += '<div id="headers" class="msgh2" style="display:none;">\n'
			for (k, v) in self.extra_headers:
				#output += sanitizeGtk(k + ':' + v.decode('latin-1')) + '<br />\n'
				mb.insert_with_tags(curr_iter, sanitizeGtk(k) + ': ', bold_tag)
				mb.insert_with_tags(curr_iter, sanitizeGtk(v.decode('latin-1')) + '\n', normal_tag)
			#next k,v
			
			mb.insert_with_tags(curr_iter, '\n', normal_tag)
			mark_end_headers = mb.create_mark("end_extra_headers", curr_iter, True)
			self.showHeader = False
			return
		#end if
		# Remove the extra headers.
		curr_iter = mb.get_iter_at_mark(mb.get_mark("extra_headers"))
		end_iter  = mb.get_iter_at_mark(mb.get_mark("end_extra_headers"))
		# Remove all text.
		mb.delete(curr_iter, end_iter)
		self.showHeader = True
	#end def clickHeader(tag, widget, event, iterc)
	
	def drawDir(self, path, currentDir = '', page = 0, msg_store=None):
		if msg_store == None:
			print("Error: drawDir called incorrectly, always need msg_store")
			return False
		#end if
		pagesize = 220
		self.currentDir = currentDir
		
		index = getIndex(currentDir, page)
		
		try:
			junkFiles = file('/tmp/mail1_junk.txt', 'r').read().split('\n')
		except:
			junkFiles = []
		#end try

		# messages on the top right
		#output = '<div class="msgsd pre">\n'
		#output += '<table cellpadding="0" cellspacing="0">\n'
		#output += '<tr><th>Subject</th><th>From</th><th>Date</th></tr>\n'
		start = time.time()
		messages = mail1.readDir(path + os.sep + currentDir + os.sep + 'cur')
		show = messages['files']
		self.messages = show
		show.sort(reverse = True)
		show = show[page*pagesize:(page+1)*pagesize]
		curr = time.time()
		print('readdir, sort dt:', curr - start)
		msgs_rows = []
		indexed = 0
		for message in show:
			if message not in index:
				fname = path + os.sep + currentDir + os.sep + 'cur' + os.sep + message
				f = file(fname, 'r')
				# This takes too long, need to use index or something...
				msg = email.message_from_file(f)
				# Write to the index
				msg_dict = {}
				msg_dict['subject'] = msg['subject']
				msg_dict['from']    = msg['from']
				msg_dict['to']      = msg['to']
				msg_dict['date']    = msg['date']
				if 'X-Spam-Level' in msg_dict:
					msg_dict['X-Spam-Level'] = msg.get('X-Spam-Level', '')
				#end if
				if 'Received-SPF' in msg_dict:
					msg_dict['Received-SPF'] = msg.get('Received-SPF', '')
				#end if
				# A fairly cheap index.
				index[message] = msg_dict
			else:
				fname = path + os.sep + currentDir + os.sep + 'cur' + os.sep + message
				indexed += 1
				# Pull from the index
				msg = index[message]
			#end if
			#msg = {'subject':'', 'from':'', 'date':''}
			subject = msg['subject']
			lsubject = decodeHeaderGtk(subject, False)
			mfrom = msg['from']
			#try:
			#	fenc = email.header.decode_header(mfrom)[0][0]
			#except:
			#	pass
			#end try
			#mfrom = sanitize(mfrom)
			fenc = decodeHeaderGtk(mfrom, False)
			#fenc = fenc.decode('latin-1')
			#if type(fenc) == str:
			#	fenc = str(fenc.__class__)
			#end if
			mfrom = sanitizeGtk(fenc)
			date = sanitizeGtk(msg['date'])
			spam_level = len(msg.get('X-Spam-Level', ''))
			if fname in junkFiles: spam_level = 10
			spf = msg.get('Received-SPF', '')
			if spf[:8].lower() == 'softfail':
				spam_level = max(spam_level, 5)
			elif spf[:4].lower() == 'none':
				spam_level = max(spam_level, 4)
			#end if
			#spam_icon = '<span style="background-color:' + mail1.spam_color[min(spam_level, len(mail1.spam_color)-1)] + '">&nbsp;</span>'
			msg_icon = self.unreadPixBuf
			if spam_level > 5:
				msg_icon = self.spamPixBuf
			elif spam_level == 5:
				msg_icon = self.fivePixBuf
			elif spam_level == 4:
				msg_icon = self.fourPixBuf
			elif spam_level == 3:
				msg_icon = self.threePixBuf
			elif spam_level == 2:
				msg_icon = self.twoPixBuf
			#end if
			smfrom = mfrom
			if len(lsubject) > 50:
				ssubject = sanitizeGtk(lsubject[:50]) + '...'
			else:
				ssubject = sanitizeGtk(lsubject)
			#end if
			lsubject = sanitizeGtk(lsubject)
			if len(fenc) > 20:
				smfrom = sanitizeGtk(fenc[:20]) + '...'
			else:
				smfrom = mfrom
			#end if
			#output += '<tr><td><div class="s"><a href="mail1mod.py?dir=' + currentDir + '&msg=' + message + '" title="' + lsubject + '" target="msgf">' + spam_icon + ssubject + '</a></div></td><td><div class="f" title="' + mfrom + '">' + smfrom + '</div></td><td><div class="d" title="' + date + '">' + date + '</div></td></tr>\n'
			msgs_rows.append([msg_icon,ssubject,smfrom,date])
		#next message
		
		threaded_rows, showThread = toThreads(msgs_rows, show)
		self.messages = showThread
		for row in threaded_rows:
			msg_store.append(row)
		#next row
		#output += '</table>'
		#output += '</div>\n'
		#output += '<a href="mail1mod.py?dir=' + currentDir + '&page=' + sanitizeGtk(str(int(page) + 1)) + '">Next page</a>\n'
		#output += '</body>'
		#output += '</html>'
		print("indexed:", indexed)
		end = time.time()
		print('render dt:', end - curr)
		saveIndex(currentDir, page, index)
		#return msg_list
	#end def drawDir(path = '/home/jvoss/Mail', currentDir = '', page = 0, msg_store=None)

	def drawMsg(self, mb, path, currentDir = '', currentMessage = '', add = '', use_part = None):
		if mb == None:
			print("Error: drawMsg called incorrectly, always need mb")
			return False
		#end if
		html = (add == 'html')
		tryCleanHtml = False
		useTextHtml = False
		if add == 'clean_html':
			html = True
			tryCleanHtml = True
		elif add == 'text_html':
			html = True
			useTextHtml = True
		#end if
		img  = (add == 'img')
		app  = (add == 'app')
		gpg  = (add == 'gpg')

		if '../' in currentDir or '../' in currentMessage:
			print("No directory traversal please...")
			mb.insert(mb.get_start_iter(), "No directory traversal please...")
			return
		#end if
		i = 0
		
		mime_ext = {'image/jpeg':'jpg', 'image/png':'png', 
		'image/gif':'gif', 'application/pdf':'pdf', 
		'application/zip':'zip', 'text/plain':'txt', 
		'application/pgp-encrypted':'txt'}
		# TODO: disallow certain extensions
		#disallow_ext = ['exe', 'sh', 'py', 'bin']
		colormap = self.window.get_colormap()
		black_color = colormap.alloc_color(0, 0, 0x0)
		# Not sure why blue is 0xffff, should be yellow.
		blue_color = colormap.alloc_color(0, 0, 0xffff)
		big_tag = mb.create_tag("big_yellow_bg", editable=False,
			foreground_gdk=black_color,
			background='yellow',
			size_points=12.0)
		normal_tag = mb.create_tag("normal", editable=False,
			foreground_gdk=black_color,
			size_points=10.0)
		spam_link_tag = mb.create_tag("spam_link", editable=False,
			foreground_gdk=blue_color,
			underline=pango.UNDERLINE_SINGLE,
			size_points=10.0)
		spam_link_tag.connect('event', self.clickSpam)
		reply_link_tag = mb.create_tag("reply_link", editable=False,
			foreground_gdk=blue_color,
			underline=pango.UNDERLINE_SINGLE,
			size_points=10.0)
		reply_link_tag.connect('event', self.clickReply)
		headers_tag = mb.create_tag("headers", editable=False,
			foreground_gdk=blue_color,
			underline=pango.UNDERLINE_SINGLE,
			size_points=10.0)
		headers_tag.connect('event', self.clickHeader)
		html_link_tag = mb.create_tag("html_link", editable=False,
			foreground_gdk=blue_color,
			underline=pango.UNDERLINE_SINGLE,
			size_points=10.0)
		html_link_tag.connect('event', self.clickHtml)
		html_text_link_tag = mb.create_tag("html_text_link", editable=False,
			foreground_gdk=blue_color,
			underline=pango.UNDERLINE_SINGLE,
			size_points=10.0)
		html_text_link_tag.connect('event', self.clickHtmlText)
		html_clean_link_tag = mb.create_tag("html_clean_link", editable=False,
			foreground_gdk=blue_color,
			underline=pango.UNDERLINE_SINGLE,
			size_points=10.0)
		html_clean_link_tag.connect('event', self.clickHtmlClean)
		plain_link_tag = mb.create_tag("plain_link", editable=False,
			foreground_gdk=blue_color,
			underline=pango.UNDERLINE_SINGLE,
			size_points=10.0)
		plain_link_tag.connect('event', self.clickPlain)
		link_tag = mb.create_tag("link", editable=False,
			foreground_gdk=blue_color,
			underline=pango.UNDERLINE_SINGLE,
			size_points=10.0)
		bold_tag = mb.create_tag("bold", editable=False,
			foreground_gdk=black_color,
			weight=pango.WEIGHT_BOLD,
			size_points=10.0)

		# message body on the bottom right
		start_iter = mb.get_start_iter()
		#mb.insert_with_tags(start_iter, )
		#iter1 = self.get_start_iter()
		#iter2 = self.get_end_iter()
		#output = '<div class="msgd">\n'
		curr_iter = start_iter
		
		if not os.path.exists(path + os.sep + currentDir + os.sep + 'cur' + os.sep + currentMessage):
			currentMessage2 = currentMessage + ':2,S'
			if not os.path.exists(path + os.sep + currentDir + os.sep + 'cur' + os.sep + currentMessage2):
				mb.insert_with_tags(start_iter, 'File not found. Please try again.', big_tag)
				return False
			else:
				currentMessage = currentMessage2
			#end if
		#end if
		
		"""
		We don't have write access, so no go. This is easy but silly.
		if currentMessage[-4:] != ':2,S' and currentMessage[-5:] != ':2,RS':
			currentMessage2 = currentMessage + ':2,S'
			# It's been seen, so rename it (this is way weird)
			os.rename(path + '/' + currentDir + '/cur/' + currentMessage, path + '/' + currentDir + '/cur/' + currentMessage2)
			currentMessage = currentMessage2
		#end if
		"""
		
		f = file(path + os.sep + currentDir + os.sep + 'cur' + os.sep + currentMessage, 'r')
		f.seek(0, 2)
		file_size = f.tell()
		f.seek(0, 0)
		msg = email.message_from_file(f)
		subject = msg['subject']
		ssubject = decodeHeaderGtk(subject)
		mfrom = decodeHeaderGtk(msg['from'])
		date = decodeHeaderGtk(msg['date'])
		mto = decodeHeaderGtk(msg['to'])
		message_id = None
		if 'Message-ID' in msg:
			message_id = decodeHeaderGtk(msg['Message-ID'])
		#end if
		mcc = None
		if 'cc' in msg:
			mcc = decodeHeaderGtk(msg['cc'])
		#end if
		from_filter = mail1.genFilter(msg['from'], 'From')
		to_filter = mail1.genFilter(msg['to'], 'To')

		#output += '<a href="#" onclick="if (window.parent && window.parent.reply) window.parent.reply(document); else alert(\'fixme \' + window.parent.toString());">Reply</a>\n'
		mb.insert_with_tags(curr_iter, 'Reply', reply_link_tag)
		mb.insert_with_tags(curr_iter, ' ', normal_tag)
		#output += '<a href="mail1mod.py?' + urllib.urlencode({'dir':currentDir, 'msg': currentMessage, 'junk':'1'}) + '">Spam</a>\n'
		mb.insert_with_tags(curr_iter, 'Spam', spam_link_tag)
		mb.insert_with_tags(curr_iter, ' ', normal_tag)

		mb.insert_with_tags(curr_iter, 'Headers', headers_tag)
		mb.insert_with_tags(curr_iter, '\n', normal_tag)
		#output += '<div class="msgh pre" onclick="if(headers.style.display==\'block\') headers.style.display=\'none\'; else headers.style.display=\'block\';">\n<div><b>Subject:</b> <span id="s">' + ssubject + '</span></div>\n'
		mb.insert_with_tags(curr_iter, 'Subject: ', bold_tag)
		mb.insert_with_tags(curr_iter, ssubject + '\n', normal_tag)
		#output += '<div><b>From:</b> <span id="f">' + mfrom + '</span> <span class="filter" onclick="this.innerHTML=unescape(\'' + from_filter + '\'); return false;">Filter</span></div>\n'
		mb.insert_with_tags(curr_iter, 'From: ', bold_tag)
		mb.insert_with_tags(curr_iter, mfrom + '\n', normal_tag)
		#output += '<div><b>To:</b> <span id="t">' + mto + '</span> <span class="filter" onclick="this.innerHTML=unescape(\'' + to_filter + '\'); return false;">Filter</span></div>\n'
		mb.insert_with_tags(curr_iter, 'To: ', bold_tag)
		mb.insert_with_tags(curr_iter, mto + '\n', normal_tag)
		if mcc:
			#output += '<div id="cc"><b>Cc:</b> <span id="c">' + mcc + '</span></div>\n'
			mb.insert_with_tags(curr_iter, 'Cc: ', bold_tag)
			mb.insert_with_tags(curr_iter, mcc + '\n', normal_tag)
		#end if
		#output += '<div><b>Date:</b> <span id="d">' + date + '</span></div>\n'
		mb.insert_with_tags(curr_iter, 'Date: ', bold_tag)
		mb.insert_with_tags(curr_iter, date + '\n', normal_tag)
		if message_id:
			#output += '<div><b>Message-ID:</b> <span id="messageid">' + message_id + '</span></div>\n'
			mb.insert_with_tags(curr_iter, 'Message-ID: ', bold_tag)
			mb.insert_with_tags(curr_iter, message_id + '\n', normal_tag)
		#end if
		#output += '</div>\n'
		#return output
		#output += '<div class="pre">\n'
		mark_headers = mb.create_mark("extra_headers", curr_iter, True)
		#mb.add_mark(mark_headers, curr_iter)
		self.showHeader = True
		self.extra_headers = msg.items()
		# See clickHeader for what goes here.
		#output += '</div>\n'
		#output += '</div>\n'
		#output += '<div class="pre" id="b">\n'
		signed = False
		encrypted = False
		for part in msg.walk():
			#if part.get_size() > 10000000:
			#	output = ''
			#end if
			ctype = part.get_content_type()
			f_raw = part.get_filename()
			if f_raw == None: f_raw = ''
			if f_raw != '':
				f_decoded = email.header.decode_header(f_raw)
				if f_decoded[0][1] != None:
					f_uni = f_decoded[0][0].decode(f_decoded[0][1]).encode('utf-8')
					filename = sanitizeGtk(f_uni)
				else:
					filename = sanitizeGtk(f_decoded[0][0])
				#end if
			else:
				filename = None
			#end if
			#ssize = str(part.get_size())
			ssize = ''
			#output += sanitizeGtk(ctype) + '<br />\n'
			mb.insert_with_tags(curr_iter, sanitizeGtk(ctype) + '\n', normal_tag)
			#fname = part.get_param('name')
			#output += 'dir:' + sanitize(repr(dir(part))) + '<br />\n'
			#output += 'name:' + sanitize(repr(part.get_param('name'))) + '<br />\n'
			#continue
			if ctype == 'multipart/related' or ctype == 'multipart/mixed' or ctype == 'multipart/alternative' or ctype == 'multipart/signed' or ctype == 'multipart/encrypted':
				if ctype == 'multipart/signed':
					signed = True
				#end if
				if ctype == 'multipart/encrypted':
					encrypted = True
				#end if
				pass
				#output += '<div style="border:1px solid black;">\n'
				#for partb in part.walk():
				#	output += sanitize(str(part))
				#	output += '<hr />\n'
				#next partb
				#output += '</div>\n'
			elif ctype[:6] == 'image/':
				ext = 'bin'
				if ctype in mime_ext: ext = mime_ext[ctype]
				if filename == None: filename = ('image%03i' % i) + '.' + ext
				if img and (str(i) == use_part):
					return (ctype, part.get_payload(decode=True), None)
				else:
					#output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&img=' + str(i) + '"><img src="/images/image2.png" border="0" /> ' + str(img) + ' ' + str(use_part) + ' ' + str(i) + ' ' + filename + '</a><br />\n'
					mb.insert_with_tags(curr_iter, str(img) + ' ' + str(use_part) + ' ' + str(i) + ' ' + filename + '\n', link_tag)
				#end if
			elif ctype[:12] == 'application/' or filename != None:
				ext = 'bin'
				if encrypted:
					ext = 'gpg'
				#end if
				if ctype in mime_ext: ext = mime_ext[ctype]
				if filename == None: filename = ('file%03i' % i) + '.' + ext
				if app and (str(i) == use_part):
					return (ctype, part.get_payload(decode=True), filename)
				else:
					try:
						filenameu = unicode(filename)
					except UnicodeDecodeError:
						filenameu = repr(filename)
					#end try 
					#output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&app=' + str(i) + '"><img src="/images/binary.png" border="0" /> ' + filenameu + '</a> ' + ssize + '<br />\n'
					mb.insert_with_tags(curr_iter, filenameu, link_tag)
					mb.insert_with_tags(curr_iter, ' '  + ssize + '\n', normal_tag)
				#end if
			elif ctype[:6] == 'audio/':
				ext = 'bin'
				if ctype in mime_ext: ext = mime_ext[ctype]
				if filename == None: filename = ('audio%03i' % i) + '.' + ext
				if app and (str(i) == use_part):
					return (ctype, part.get_payload(decode=True), filename)
				else:
					try:
						filenameu = unicode(filename)
					except UnicodeDecodeError:
						filenameu = repr(filename)
					#end try 
					mb.insert_with_tags(curr_iter, filenameu, link_tag)
					#output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&app=' + str(i) + '"><img src="/images/binary.png" border="0" /> ' + filename + '</a><br />\n'
				#end if
			elif ctype == 'text/html':
				if html == True:
					# Be very careful to ensure this does not get execed.
					pdata = part.get_payload(decode=True)
					ct = part.get('Content-Type')
					charset = 'ascii'
					if ct and 'charset=' in ct:
						charset = ct[ct.index('charset=')+8:]
						try:
							pdata = pdata.decode(charset)
						except LookupError as le:
							# TODO: warning.
							pass
						#end try
					#end if
					try:
						uni_input = part.get_payload(decode=True).decode(charset)
					except LookupError:
						# FIXME: This could be better handled.
						uni_input = part.get_payload(decode=True).decode('ascii')
					#end try
					if tryCleanHtml:
						#output += mail1.clean_html(uni_input)
						# TODO GTK HTML?
						mb.insert_with_tags(curr_iter, mail1.clean_html(uni_input), normal_tag)
					elif useTextHtml:
						#output += sanitizeGtk(text_html(uni_input)).replace('\n', '<br />\n')
						mb.insert_with_tags(curr_iter, mail1.text_html(uni_input), normal_tag)
					else:
						#output += sanitizeGtk(uni_input).replace('\n', '<br />\n')
						mb.insert_with_tags(curr_iter, sanitizeGtk(uni_input), normal_tag)
					#end if
					# TODO: GTK 
					#output += '<hr />\n'
				else:
					if filename == None:
						filename = ''
					elif filename != '':
						filename = ' ' + filename
					#end if
					#output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&html=1">html ' + filename + '</a><br />\n'
					mb.insert_with_tags(curr_iter, 'html' + filename + '\n', html_link_tag)
					#output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&text_html=1">html text ' + filename + '</a><br />\n'
					mb.insert_with_tags(curr_iter, 'html text' + filename + '\n', html_text_link_tag)
					#output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&clean_html=1">html clean ' + filename + '</a><br />\n'
					mb.insert_with_tags(curr_iter, 'html clean' + filename + '\n', html_clean_link_tag)
				#end if
			elif ctype == 'text/plain':
				if (app or gpg) and (str(i) == use_part):
					ext = 'txt'
					if filename == None: filename = ('file%03i' % i) + '.' + ext
					if gpg:
						# RFC3156 wants unencoded with headers.
						#output = '\n'.join([': '.join(a) for a in part.items()]) + '\n\n'
						#output += part.get_payload(decode=False)
						mb.insert_with_tags(curr_iter, '\n'.join([': '.join(a) for a in part.items()]) + '\n\n', normal_tag)
						mb.insert_with_tags(curr_iter, part.get_payload(decode=False), normal_tag)
					else:
						#output = part.get_payload(decode=True)
						mb.insert_with_tags(curr_iter, part.get_payload(decode=True), normal_tag)
					#end if
					# TODO: GTK Create a file and save it somewhere.
					#return (ctype, output, filename)
				if html == False:
					#.decode()
					pdata = part.get_payload(decode=True)
					ct = part.get('Content-Type')
					charset = 'ascii'
					if ct and ('charset=' in ct or 'charset =' in ct):
						if 'charset=' in ct:
							charset = ct[ct.index('charset=')+8:].strip()
						elif 'charset =' in ct:
							charset = ct[ct.index('charset =')+9:].strip()
						if ';' in charset: charset = charset[:charset.index(';')]
						if '"' in charset: charset = charset.replace('"','')
						try:
							pdata = pdata.decode(charset)
						except LookupError as le:
							# TODO: warning.
							pass
						except UnicodeDecodeError as ude:
							# TODO: warning.
							#pdata = pdata.decode('utf-8')
							pass
						#end try
					#end if
					# Also be very careful to ensure this does not get execed.
					#if msg['Content-Transfer-Encoding'] == 'base64':
					#	pdata2 = pdata.split('\n\n', 1)
					#	pdata = pdata2[0] + base64.b64decode(pdata2[1])
					#end if
					#output += charset + sanitize(repr(pdata))
					try:
						#output += sanitizeGtk(pdata).replace('\n', '<br />\n')
						mb.insert_with_tags(curr_iter, sanitizeGtk(pdata) + '\n', normal_tag)
					except UnicodeDecodeError as eex:
						#output += 'Error with data: ' + str(eex) + ' <br />\n' + repr(part.get_params()) + ' <br />\n' + sanitizeGtk(repr(pdata))
						mb.insert_with_tags(curr_iter, 'Error with data: ' + str(eex) + '\n' + repr(part.get_params()) + '\n' + sanitizeGtk(repr(pdata)) + '\n', normal_tag)
					# + "(" + str(charset) + " " + repr(part.get_all('Content-Type')) + ", " + str(type(pdata)) + ")").replace('\n', '<br />\n')
					# TODO: GTK 
					#output += '<hr />\n'
					if signed:
						#output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&gpg=' + str(i) + '">gpg</a><br />\n'
						mb.insert_with_tags(curr_iter, 'gpg' + '\n', link_tag)
					#end if
					#output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '&app=' + str(i) + '">plaintext</a><br />\n'
					mb.insert_with_tags(curr_iter, 'plaintext' + '\n', link_tag)
				else:
					#output += '<a href="mail1mod.py?dir=' + currentDir + '&msg=' + currentMessage + '">text</a><br />\n'
					mb.insert_with_tags(curr_iter, 'text' + '\n', plain_link_tag)
				#end if
			else:
				try:
					body = sanitizeGtk(str(part)) + '\n'
				except UnicodeDecodeError:
					body = sanitizeGtk(str(part).decode('utf-8')) + '\n'
				#end try
				mb.insert_with_tags(curr_iter, body, normal_tag)
				#output += '<hr />\n'
			#end if
			#if len(output) > 1000000:
			#	output += "<div>Debug: It's a big one</div>\n"
			#end if
			#i += 1
		#next part
		#output += '</div>\n' # pre
		#end if
		#output += '</div>\n'
		
		#output += '</body>'
		#output += '</html>'
		return True #output
	#end def drawMsg()

	def showAll(self):
		"""
		Calls self.window.show_all() and gtk.main()
		"""
		self.window.show_all()
		gtk.main()
	#end def showAll()
#end class MaildirWindow

def main():
	maildir = os.environ['HOME'] + os.sep + 'Mail'
	w = MaildirWindow(maildir)
	w.showAll()
#end def main()

if __name__ == '__main__':
	main()
#end if
